/* 89-03-02 Jahns	_kbbgd() with two parameter			*/
/*				0 : wait for keyboard input		*/
/*				1 : no wait for keyboard input		*/


/* (c) 1985, Phoenix Computer Products Corp. and Novum Organum, Inc. */
/*
* name : 	_kbbgd - executes background tasks while waiting for
*			keyboard input.
*
* synopsis : 	_kbbgd(pkb_check)
*			PFI	pkb_check;
*
*		(*pkb_check)() is a pointer to a function which checks
*		to see if a character is waiting at the keyboard input.
*
* description :	_kbbgd() takes a pointer to a function which it uses
*		to check if a key is waiting at the keyboard.  
*		There are two 
*		arrays of pointers to functions.  if nothing is currently
*		waiting at the keyboard, _kbbgd() begins to loop through
*		the arrays.  Any non-NULL values are considered to be
*		pointers for 'background' functions.  Each
*		function is called once unless the user presses a key.
*		In that case, the looping is terminated and the key
*		is returned to the function which called kbnec().
*
* version	1.0 (c) novum organum, inc. 1985
**/
#include "compiler.h"
#include "tskenv0.h"
#include "cptpdf.h"

#define	NFNCS	10
#undef	NULL
#define	NULL	0

static	PFI _bgdtask[NFNCS]={NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};

#ifdef use

static	PFI _sysbgdtask[NFNCS]={NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};

static int	sys_cnt = NFNCS;
static int	bgd_cnt = NFNCS;
static PFI	*sys, *bgd;

extern struct tsk_env	*TSKENV;



_kbbgd (PFI pkb_check, int term)
{

/*    while(!(*pkb_check)()) */
  for(;;)
  {
    if	(sys_cnt >= NFNCS)
    {
      sys = _sysbgdtask;
      sys_cnt = 0;
    }
    if	(bgd_cnt >= NFNCS)
    {
      bgd  = _bgdtask;
      bgd_cnt = 0;
    }

    if	(*bgd)		    /* perform any programmer defined backgd fncs */
	(*(*bgd))();
    bgd++;		    /* step through programmer's array      */
    ++bgd_cnt;

    if (! TSKENV->PKBRD->BATCH)
    {
      if  ((*pkb_check)())    /* if user pressed key, then quit       */
	  break;
      if  (*sys)	      /* perform any system background function */
	  (*(*sys))();
      sys++;		      /* step through the array 	      */
      ++sys_cnt;
    }
    else {
      if  (*sys)	      /* perform any system background function */
	  (*(*sys))();
      sys++;		      /* step through the array 	      */
      ++sys_cnt;
      if  ((*pkb_check)())    /* if user pressed key, then quit       */
	  break;
    }

    if (term) break;
  }
}

#endif

/***
*	internal function to set pointers in background-function arrays.
***/

static PFI _kbsetbgd (bgd_ary, i, bgdtask)
    PFI		*bgd_ary, bgdtask;
    int   	i;
{
    PFI		pfi;

    if	((0 <= i) && (i < NFNCS))	/* if array elm number is ok 	*/
    	{				/* then */
    	pfi = bgd_ary[i];		/* get current pointer		*/
    	bgd_ary[i] = bgdtask;		/* add the new pointer		*/
    	}
    else    
	pfi = NULLPFI;
    return (pfi);	/* return the value that was previously in array*/
}

/*
* name : 	kbsetbgd - set a background task.
*
* synopsis : 	old_pfi = kbsetbgd (i, bgdtask)
*			int	i;
*			PFI	bgdtask;
*
* description :	kbsetbgd() sets the I'th slot in the background-task
*		array to BGDTASK.  BGDTASK should be a pointer to a
*		function which takes no parameters.  this function
*		will be called while kbdosnec() is waiting for input.
*		I must be a value between 0 and NFNCS-1.  
*
* returns	pfi	the value of the function pointer previously stored
*			int the I'th slot.  this allows you to restore
*			the previous pointer later.
*		NULLPFI	is returned if no function pointer was stored.
*
* version	1.0 (c) novum organum, inc. 1985
**/

PFI kbsetbgd (i, bgdtask)
    int   	i;
    PFI		bgdtask;
{
    return (_kbsetbgd (_bgdtask, i, bgdtask));
}

#ifdef use

/***
*	kbsetsysbgd() is reserved for internal use by the libraries
*	to setup back-ground tasks, e.g. slot 0 is used by the
*	word-processor library for its screen update task.
*
***/

PFI kbsetsysbgd (i, bgdtask)
    int   	i;
    PFI		bgdtask;
{
    return (_kbsetbgd (_sysbgdtask, i, bgdtask));
}
#endif
