/*	pfdb.h		*/

#define PGSIZ		2048	   /* database page size */

#define RH_DBFHDR 	1L	/* record handle of file header */
#define RH_PGTBL	513L	/* record handle of the page map */

#define CELL_CONTAINS_DATA	0x4000
#define	ENDFREE		0
#define FARCELL		0x8000
#define	FARCELLSIZ	(sizeof(int)+sizeof(HDL32))
#define	F_IN_TRNSACT	0x02
#define	F_NEW_PGS	0x01
#define FOOTERSIZ	24
#define	HIPGMASK	0x00ff0000
#define	IN_LFC		0xf000
#define MAXRECSIZ	(PGSIZ-FOOTERSIZ-(3*sizeof(int)))
#define	N_IDBITS		9
#define ONDISK		0xf000
#define	RH_IDMASK	0x01ff
#define	RH_PGMASK	0x7fff
#define	RSVDID		0xfff0
#define	SIZPART		0x0fff
#define UNUSED		0xffff

/*	Record types for predefined database records.	*/

#define RT_DBFHDR  	0x8001	/* database file header record.	*/
#define	RT_PGMAP 	0x8002	/* page map record.		*/
#define	RT_BHDR		0x8003	/* b-tree header record.	*/
#define RT_BNODE 	0x8004	/* b-tree node record.		*/

/*
*	the pages look as follows :
*	
*	[ <data>      byte 0
*	   |
*	   V downward 	    
*
*	  ^ upward
*	  |	  
*	cell list  
*	<ncells>		
*	<free area>
*/

/*	THIS STRUCTURE MUST NOT EXCEED ONE PARARGAPH IN LENGTH */

#define	CELLTBL	struct _CELLTBL
#define	H_CELLTBL	((CELLTBL *)h_celltbl)

CELLTBL	{
	short	ncells,
		vector_siz,
		lfc;	/*list of free cells*/
	ushort	free_off,
		free_sp;
	};

#define	PGVARS	struct _PGVARS
#define	H_PGVAR	((PGVARS *)h_pgvar)

PGVARS	{
	HDL	h_buf;
	short	cell_id,
		data_siz;
	short	expd_cell_siz;
	char	*pg;
	short	*vector;
	HDL	h_celltbl;
	short	*vector_elm;
	short	*pcell;
	short	cell_siz;
	RHDL 	rh_far_cell;
	};

#define	PGMAP	struct	_PGMAP
#define	H_PGMAP	((PGMAP *)h_pgmap)

PGMAP	{
	short	rec_typ;		/* record type of stored pg tables */
	short	max_pgs;
	short	lst_pg;
	RHDL	rh_nxt_pgmap;
	long	dummy2;		  
	long	dummy3;
	ushort	map_free_sp[1];	
	};


#define	DBFHDR	struct _DBFHDR
#define	H_DBFHDR	((DBFHDR *)h_dbfhdr)

DBFHDR	{
	short	rec_typ; 
	short	open_flags;
	RHDL	rh_anchor,
		rh_pgtbl;	/* free sector map */
	short	versn,
		rlse;
	short	pg_siz;
	long	create_dttm,
		upd_dttm;
	long	dum1;
	long	dum2;
	};

#define	xc	"Proprietary format of Novum Organum, Inc. 1985"

#define	LEAF 		127
#define	BRANCH 		0
#define	DUPKEYS		0x8000
#define MAXDEPTH	15

/* If you are using the B-trees as multiuser indexes, then you cannot */
/* merge nodes during deletion because this destroys the context of */
/* some of the processes.  Thus some nodes are left less than half  full. */
/* If you are using the trees as single user, define SINGLEUSER in */
/* DBCFG.H, which adds code for merging of nodes on deletion */

#define	SINGLEUSER	 

/*	btc	- btree cell 	*/

#define	BTC	struct _BTC
#define	H_BTC	((BTC *)h_btc)

BTC	{
	HDL	h_nxtbtc,
		h_prvbtc;
	short	btc_tag;
	RHDL	rh_bhdr;	/* record HDL of btree header */
	HDL	h_db,		/* HDL of btree's database */
		h_data_db;	/* HDL of data record's file or database*/
	short	open_ct;
	RHDL	rh_root;	/* record HDL of current root */
	long	keyct;		/* number of keys in this btree */
	HDL	pkeys;		/* ptr to key list used for comparisons */
    	short	keysiz,		/* size of entire key */
		keytyp;		/* type of first portion of key */
	int	f_dupkeys;	/* if yes, then duplicate keys are allowed */
	short 	max_depth,	/* maximum depth of the stack */
		depth;		/* stack depth for this search */
	RHDL	rh_node [MAXDEPTH];/*stack*/
	};

#define	BTR	struct _BTR
#define	H_BTR	((BTR *)h_btr)

BTR	{
	HDL	h_nxtbtr,
		h_prvbtr;
	short	btr_tag;
	HDL	h_btc;
	RHDL	rh_actv_leaf;	/* record HDL of leaf containing */
				/* the active key */
	char	*actv_key;	/* number of key in active leaf which */
				/* is the active key */
	char	f_actv_prcd;	/* if yes, the active key has been processed*/
	char	f_actv_exists;	/* if yes, there is an active key.	*/
	};


#define	BNODE	struct _BNODE
#define	H_BNODE	((BNODE *)h_bnode)

BNODE	{
	ushort	bnrec_typ;
	short	node_typ,
		max_bkeys,
		bnkey_typ,
		bnkey_siz,
		o_bkeys,
		bnkey_ct;
	long	rh_lf_sibling,
		rh_rt_sibling;
	short	f_lockd;
	short	bnflags,	/* bit 0x8000 = allow duplicate keys */
		bndum1;
	long	bndum2,bndum3;
	char	keys[sizeof(long)];
	};

/**
*	This holds the header for a btree.  It contains a count
*	of the keys in the tree, but more important is contains
*	the HDL of the btree root.
**/

#define	BHDR	struct _BHDR
#define	H_BHDR	((BHDR *)h_bhdr)

BHDR	{
	ushort	bhrec_typ;	/* 'bh' - because ci-86 needs unique names */
	RHDL	bhrh_root;
	long	bhkey_ct;
	long	bhcreate_dttm,
		bhrefrnc_dttm;
	short	bhbtr_versn;
	ushort	bhflags;
	short	bhdum1;
	long	bhdum2,
		bhdum3;
	short	bhnkeys;
	short	bhkey_info[1];
	};
