/* (c) 1985, Phoenix Computer Products Corp. and Novum Organum, Inc. */
#include "cptpdf.h"
#include "psys.h"

static  int     _vmode;
static  int     _cmode;
static  int     _page;
static  ushort  _oldseg, _oldoff;   /* old video interrupt service */

static _winttty();
static _wintlpen();
static _wintwrdot();
static ushort _wintrddot();
static _wintscr();
static _wintstr();

/***
* Name:         wintinit -- install WINT as the video interrupt service
*
* Synopsis:     void    wintinit();
*
*               wintinit();
*
* Description:  called by wndstart() to set up
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*
***/
/*VOID*/wintinit()
{
	REGIS	regs;

    regs.ax = TOHIBYTE(V_GETVIDSTATE);
    syivexe(V_VID_INTRPT,&regs);
    _vmode = LOBYTE(regs.ax);

    _page = HIBYTE(regs.bx);
    _wndmode( _vmode );

    regs.ax = TOHIBYTE(V_GETCP);   /* get cursor type and position */
    syivexe(V_VID_INTRPT,&regs);
    _cmode = regs.cx;              /* save cursor type */

    syivget( V_VID_INTRPT, &_oldseg, &_oldoff );
    _wndinit( _oldseg, _oldoff );
    _wndmode( _vmode );

}


/***
* Name:         wint -- implements the BIOS video services
*
* Synopsis:     void    wint();
*               	REGIS	*regs;
*
*               wint( &regs );
*
* Description:  Does all the same things that the original BIOS did for
*               the video interrupt, but interprets the BIOS requests in
*               light of the window system.  This is called internally
*               by the assembly routine _WNDINT.  _WNDINT is installed in
*               place of the original BIOS interrupt routine for INT 10H
*               by _WNDINIT.
*
* Returns:      nothing, but may alter REGS appropriately
*
* (C) Novum Organum, Inc. 1985
*
***/
/*VOID*/wint( regs )
	REGIS	*regs;
{
    ushort  _wintrddot();
    int prow, pcol, astate;

    switch  ( HIBYTE(regs->ax) )        /* switch on function type  */
	{
    case 0:                             /* CHANGE VIDEO MODE        */
        _vmode = LOBYTE(regs->ax);
        _wndmode( _vmode );             /* note change */
        syisrexe(_oldseg,_oldoff,regs); /* let original BIOS do it  */
        wndreset();                     /* redraw everything        */
        break;
    case 1:                             /* RESET CURSOR TYPE        */
        _cmode = regs->cx;              /* note change              */
        syisrexe(_oldseg,_oldoff,regs); /* let original BIOS do it  */
        break;
    case 2:                             /* LOCATE CURSOR            */
        _wasetloc((int)HIBYTE(regs->dx),(int)LOBYTE(regs->dx));
        _waloc();
        break;
    case 3:                             /* RETURN CURSOR POSITION   */
        _wagetloc(&prow,&pcol);
        regs->dx = MAKEWORD(prow,pcol);
        regs->cx = _cmode;              /* also return cursor type  */
        break;
    case 4:                             /* CHECK LIGHT PEN          */
        _wintlpen( regs );
        break;
    case 5:                             /* CHANGE ACTIVE PAGE       */
        _page = LOBYTE(regs->ax);       /* windows have no pages    */
        break;
    case 6:                             /* SCROLL UP                */
        wbscrup( regs );                /* scroll buffer            */
        _waupd();                       /* redraw whole window      */
        break;
    case 7:                             /* SCROLL DOWN              */
        wbscrdn( regs );                /* scroll buffer            */
        _waupd();                       /* redraw whole window      */
        break;
    case 8:                             /* GET CHAR AND ATTR        */
        _wagetloc(&prow,&pcol);
        regs->ax = wbgetca(prow,pcol);
        break;
    case 9:                             /* WRITE CHAR AND ATTR      */
        _wawrite(VD_1C1A,(int)regs->cx,(char *)&regs->ax,(int)LOBYTE(regs->bx));
        break;
    case 10:                            /* WRITE CHAR               */
        _wawrite(VD_1C0A,(int)regs->cx,(char *)&regs->ax,(int)LOBYTE(regs->bx));
        break;
    case 11:                            /* SET PALETTE              */
        syisrexe(_oldseg,_oldoff,regs);
        break;
    case 12:                            /* WRITE DOT                */
        _wintwrdot( (int)regs->dx, (int)regs->cx, (int)LOBYTE(regs->ax));
        break;
    case 13:                            /* READ DOT                 */
        regs->ax = _wintrddot( (int)regs->dx, (int)regs->cx );
        break;
    case 14:                            /* WRITE IN TTY MODE        */
        _winttty( VD_1C0A, (int)LOBYTE(regs->ax), (int)LOBYTE(regs->bx) );
        break;
    case 15:                            /* GET CURRENT VIDEO STATE  */
        regs->ax = MAKEWORD(wbncols(),_vmode);
        regs->bx = TOHIBYTE(_page);
        regs->cx = wbnrows();
        break;
    case 19:
        _wintstr((int)LOBYTE(regs->ax),(int)HIBYTE(regs->dx),
		 (int)LOBYTE(regs->dx),(int)regs->cx, (int)regs->es, 
		 (int)regs->bp, (int)LOBYTE(regs->bx) );
        break;
    default :
	return(RC_FAIL);    /* to signal that the interrupt is not complete */
    }
    return(RC_SUCCESS);     /* to signal that the interrupt is complete */
}


/***
* Name:         _winttty -- writes characters for the TTY service
*
* Synopsis:     void    _winttty();
*               int     mode, chr, attr;
*
*               _winttty(mode,chr,attr);
*
* Description:  handles BIOS service 14
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*
***/
static /*VOID*/_winttty( mode, chr, attr )
    int mode, chr, attr;
{
    int row, col;
    char	c;

    _wagetloc( &row, &col );

    switch  ( chr )
        {
    case BELL:
        sybeep();
        break;
    case BKSP:
        if  ( col > 0 )
            col--;
        break;
    case CRET:
        col = 0;
        break;
    case LNFEED:
        row++;
        break;
    default:
	c = chr;
        _wawrite( mode,  1, &c, attr );
        col++;
        break;
        }

    if  ( col >= wbncols() )
        {
        col = 0;
        row++;
        }

    if  ( row >= wbnrows() )
        {
        _wintscr();
        row = wbnrows() - 1;
        }
    _wasetloc( row, col );
    _waloc();
}


/***
* Name:         _wintlpen -- translates light pen coords to virtual
*
* Synopsis:     void    _wintlpen();
*               REGIS	*regs;
*
*               _wintlpen( &regs );
*
* Description:  Checks light pen for triggering -- translates position
*               to window relative position.
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*
***/
static /*VOID*/_wintlpen( regs )
	REGIS	*regs;
{
    int prow, pcol;

    syisrexe(_oldseg,_oldoff,regs);

    if  ( HIBYTE(regs->ax) )        /* if pen was triggered     */
        {
                                    /* if position not in window*/
        if  ( !wmtest((int)HIBYTE(regs->dx),(int)LOBYTE(regs->dx)) )
            regs->ax = LOBYTE(regs->ax);/* pen not triggered    */
        else
            {
            _wavpoff(&prow,&pcol);
            regs->bx -= pcol << 3;
            regs->cx -= TOHIBYTE(prow << 3);
            regs->dx -= MAKEWORD(prow,pcol);
            }
        }
}


/***
* Name:         _wintwrdot -- writes a dot
*
* Synopsis:     void    _wintwrdot();
*
*               _wintwrdot();
*
* Description:  Handles BIOS service 12.
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*
***/
static /*VOID*/_wintwrdot( row, col, attr )
    int row, col, attr;
{
    int     prow, pcol;
    REGIS   regs;

    if  ( wndgetupd() )
        {
        _wavpoff( &prow, &pcol );
        row += (prow<<3);
        col += (pcol<<3);
        if  ( wmtest(row>>3,col>>3) )
            {
            regs.ax = TOHIBYTE(V_PUTPIX);
            regs.bx = attr;
            regs.cx = col;
            regs.dx = row;
            syisrexe( _oldseg, _oldoff, &regs );
            }
        }
}

/***
* Name:         _wintrddot -- reads a dot
*
* Synopsis:     ushort _wintrddot();
*               int dot, row, col;
*
*               dot = _wintrddot( row, col );
*
* Description:  Handles BIOS service 13.
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*

***/
static ushort _wintrddot( row, col )
    int     row, col;
{
    int     prow, pcol;
    REGIS   regs;

    _wavpoff( &prow, &pcol );
    row += (prow<<3);
    col += (pcol<<3);
    regs.ax = TOHIBYTE(V_GETPIX);
    if  ( wmtest(row>>3,col>>3) )
        {
        regs.cx = col;
        regs.dx = row;
        syisrexe( _oldseg, _oldoff, &regs );
        }
    return  ( LOBYTE(regs.ax) );
}


/***
* Name:         _wintscr -- scroll the whole window
*
* Synopsis:     void    _wintscr();
*               int     nlines;
*
*               _wintscr( nlines );
*
* Description:  Scroll the screen for TTY-mode.
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*
***/
static  /*VOID*/_wintscr()
{
    REGIS   regs;

    regs.ax = 1;
    regs.bx = TOHIBYTE(_vdspattr());
    regs.cx = 0;
    regs.dx = MAKEWORD(wbnrows()-1,wbncols()-1);
    wbscrup( &regs );
    _waupd();
}


/***
* Name:         _wintstr -- writes a string
*
* Synopsis:     void    _wintstr();
*               int     mode, row, col, nchars, seg, off, attr;
*
*               _wintstr( mode, row, col, nchars, seg, off, attr );
*
* Description:  Handles BIOS service 19.
*
* Returns:      nothing
*
* (C) Novum Organum, Inc. 1985
*
***/
static  /*VOID*/_wintstr( mode, row, col, nchars, seg, off, attr )
    int mode, row, col, nchars, seg, off, attr;
{
    int orow, ocol, chrattr;

    _wagetloc(&orow,&ocol);
    _wasetloc( row, col );

    while   (nchars--)
        {
        mempf( (char *)&chrattr, (ushort)seg, (ushort)off++, 2 );
        if  ( mode&2 )
            {
            off++;
            attr = HIBYTE(chrattr);
            }
        _winttty( VD_1C1A, LOBYTE(chrattr), attr );
        }

    if  ( !(mode&1) )
        {
        _wasetloc( orow, ocol );
        _waloc();
        }
}

ushort _wintoseg()
{
    return( _oldseg );
}

ushort _wintooff()
{
    return( _oldoff );
}

